@@ -1,7 +1,8 @@
+from typing import Dict
+
 import ccxt
 import pandas as pd
 from quick_trade import utils
-from typing import Dict
 
 
 class TradingClient(object):
@@ -100,4 +101,7 @@ class TradingClient(object):
 
     @utils.wait_success
     def _update_balances(self):
-        self.start_balance = self.client.fetch_free_balance()
+        if len(self.client.apiKey):
+            self.start_balance = self.client.fetch_free_balance()
+        else:
+            utils.logger.warning('Client doesn\'t have an API key')
