@@ -9,22 +9,24 @@
 #   scalper and dca bot
 #   unit-tests
 #   more docs and examples
+#   make normal logs
 
-import time
-import typing
-from typing import Dict, List, Tuple, Any, Iterable
+from copy import copy
+from datetime import datetime
+from threading import Thread
+from time import ctime, sleep, time
+from typing import Dict, List, Tuple, Any, Iterable, Union, Sized
 
 import numpy as np
 import pandas as pd
-import quick_trade.brokers as brokers
 import ta
 import ta.momentum
 import ta.others
 import ta.trend
 import ta.volatility
 import ta.volume
-import talib
 from plotly.subplots import make_subplots
+from quick_trade import brokers
 from quick_trade import utils
 
 Line = dict  # To avoid the deprecation warning
@@ -45,7 +47,6 @@ class Trader(object):
     interval: str
     __exit_order__: bool = False
     _prev_predict: str = 'Exit'
-    _regression_inputs: int
     _stop_loss: float
     _take_profit: float
     _open_price: float
@@ -62,7 +63,7 @@ class Trader(object):
     _backtest_out_no_drop: pd.DataFrame
     backtest_out: pd.DataFrame
     _open_lot_prices: List[float]
-    realtime_returns: Dict[str, Dict[str, typing.Union[str, float]]]
+    realtime_returns: Dict[str, Dict[str, Union[str, float]]]
     client: brokers.TradingClient
     __last_stop_loss: float
     __last_take_profit: float
@@ -610,7 +611,7 @@ class Trader(object):
         data_high: pd.Series = self.df['High']
         data_low: pd.Series = self.df['Low']
         self.deposit_history = [deposit]
-        seted_ = utils.set_(self.returns)
+        seted_ = utils.convert(self.returns)
         self.trades = 0
         self.profits = 0
         self.losses = 0
@@ -722,7 +723,7 @@ winrate: {self.winrate}%"""
              self._open_lot_prices, data_column, self._linear, self.returns_strategy_diff),
             index=[
                 f'deposit ({column})', 'stop loss', 'take profit',
-                'predictions', 'open deal/lot', column,
+                'predictions', 'open trade', column,
                 f"linear deposit data ({column})",
                 "returns"
             ]).T
@@ -767,7 +768,7 @@ winrate: {self.winrate}%"""
                     y=self._open_lot_prices,
                     line=dict(width=utils.TAKE_STOP_OPN_WIDTH, color=utils.BLUE),
                     opacity=utils.STOP_TAKE_OPN_ALPHA,
-                    name='open lot'),
+                    name='open trade'),
                 row=1,
                 col=1)
             self.fig.add_trace(
@@ -776,12 +777,12 @@ winrate: {self.winrate}%"""
                     line=dict(color=utils.COLOR_DEPOSIT),
                     name=f'deposit (start: ${money_start})'), 2, 1)
             self.fig.add_trace(Line(y=self._linear, name='linear'), 2, 1)
-            preds: Dict[str, List[typing.Union[int, float]]] = {'sellind': [],
-                                                                'exitind': [],
-                                                                'buyind': [],
-                                                                'bprice': [],
-                                                                'sprice': [],
-                                                                'eprice': []}
+            preds: Dict[str, List[Union[int, float]]] = {'sellind': [],
+                                                         'exitind': [],
+                                                         'buyind': [],
+                                                         'bprice': [],
+                                                         'sprice': [],
+                                                         'eprice': []}
             for e, i in enumerate(seted_):
                 if i == utils.SELL:
                     preds['sellind'].append(e)
@@ -820,7 +821,7 @@ winrate: {self.winrate}%"""
         return self.backtest_out
 
     def multi_backtest(self,
-                       tickers: typing.Union[typing.Sized, typing.Iterable[str]],
+                       tickers: Union[Sized, Iterable[str]],
                        strategy_name: str = 'strategy_macd',
                        strategy_kwargs: Dict[str, Any] = {},
                        deposit: float = 10_000.0,
@@ -1036,7 +1037,7 @@ winrate: {self.winrate}%"""
     @staticmethod
     def _collide_super(l1, l2) -> utils.PREDICT_TYPE_LIST:
         return_list: utils.PREDICT_TYPE_LIST = []
-        for first, sec in zip(utils.set_(l1), utils.set_(l2)):
+        for first, sec in zip(utils.convert(l1), utils.convert(l2)):
             if first is not np.nan and sec is not np.nan and first is not sec:
                 return_list.append(utils.EXIT)
             elif first is sec:
@@ -1045,7 +1046,7 @@ winrate: {self.winrate}%"""
                 return_list.append(sec)
             else:
                 return_list.append(first)
-        return list(map(lambda x: utils.PREDICT_TYPE(x), utils.anti_set_(return_list)))
+        return list(map(lambda x: utils.PREDICT_TYPE(x), utils.anti_convert(return_list)))
 
     def multi_strategy_collider(self, *strategies, mode: str = 'minimalist') -> utils.PREDICT_TYPE_LIST:
         self.strategy_collider(strategies[0], strategies[1], mode=mode)
@@ -1059,7 +1060,7 @@ winrate: {self.winrate}%"""
     def get_trading_predict(self,
                             bet_for_trading_on_client: float = np.inf,
                             coin_lotsize_division: bool = True
-                            ) -> Dict[str, typing.Union[str, float]]:
+                            ) -> Dict[str, Union[str, float]]:
         """
         predict and trading.
 
@@ -1080,7 +1081,7 @@ winrate: {self.winrate}%"""
         self.__last_take_profit = self._take_profits[-1]
         self.__last_credit_leverage = self._credit_leverages[-1]
         if self._prev_predict != predict or self.__prev_credit_lev != self.__last_credit_leverage:
-            utils.logger.info(f'open lot {predict}')
+            utils.logger.info(f'open trade {predict}')
             self.__exit_order__ = False
             if self.trading_on_client:
 
@@ -1107,7 +1108,7 @@ winrate: {self.winrate}%"""
         utils.logger.debug("returning prediction")
         return {
             'predict': predict,
-            'open lot price': self._open_price,
+            'open trade price': self._open_price,
             'stop loss': self.__last_stop_loss,
             'take profit': self.__last_take_profit,
             'currency close': close[-1],
@@ -1120,7 +1121,7 @@ winrate: {self.winrate}%"""
                          print_out: bool = True,
                          bet_for_trading_on_client: float = np.inf,
                          coin_lotsize_division: bool = True,
-                         ignore_exceptions: bool = True,
+                         ignore_exceptions: bool = False,
                          print_exc: bool = True,
                          wait_sl_tp_checking: float = 5,
                          limit: int = 1000,
@@ -1144,7 +1145,7 @@ winrate: {self.winrate}%"""
 
         self.realtime_returns = {}
         self.ticker = ticker
-        open_time = time.time()
+        open_time = time()
         while True:
             try:
                 self.df = self.client.get_data_historical(ticker=self.ticker, limit=limit, interval=self.interval)
@@ -1157,15 +1158,15 @@ winrate: {self.winrate}%"""
                     bet_for_trading_on_client=bet_for_trading_on_client,
                     coin_lotsize_division=coin_lotsize_division)
 
-                index = f'{self.ticker}, {time.ctime()}'
+                index = f'{self.ticker}, {ctime()}'
                 utils.logger.info(f"trading prediction at {index}: {prediction}")
                 if print_out:
                     print(index, prediction)
                 self.realtime_returns[index] = prediction
                 while True:
                     if not self.__exit_order__:
-                        if (open_time + self._sec_interval) - time.time() < wait_sl_tp_checking:
-                            time.sleep(wait_sl_tp_checking)
+                        if (open_time + self._sec_interval) - time() < wait_sl_tp_checking:
+                            sleep(wait_sl_tp_checking)
                         utils.logger.debug(f"sleep {wait_sl_tp_checking} seconds")
 
                         price = self.client.get_ticker_price(ticker)
@@ -1173,8 +1174,8 @@ winrate: {self.winrate}%"""
                         max_ = max(self.__last_stop_loss, self.__last_take_profit)
                         if (not (min_ < price < max_)) and prediction["predict"] != 'Exit':
                             self.__exit_order__ = True
-                            utils.logger.debug('exit lot')
-                            index = f'{self.ticker}, {time.ctime()}'
+                            utils.logger.info('exit trade')
+                            index = f'{self.ticker}, {ctime()}'
                             utils.logger.info(f"trading prediction exit in sleeping at {index}: {prediction}")
                             if print_out:
                                 print(f"trading prediction exit in sleeping at {index}: {prediction}")
@@ -1183,7 +1184,7 @@ winrate: {self.winrate}%"""
                                 self.client.exit_last_order()
                         elif strategy_in_sleep:
                             break
-                    if time.time() >= (open_time + self._sec_interval):
+                    if time() >= (open_time + self._sec_interval):
                         self._prev_predict = utils.convert_signal_str(self.returns[-1])
                         open_time += self._sec_interval
                         break
@@ -1197,6 +1198,61 @@ winrate: {self.winrate}%"""
                 else:
                     raise exc
 
+    def multi_realtime_trading(self,
+                               tickers: List[str],
+                               start_time: datetime,  # LOCAL TIME
+                               strategy_name: str,
+                               print_out: bool = True,
+                               bet_for_trading_on_client: float = np.inf,  # for 1 trade
+                               coin_lotsize_division: bool = True,
+                               ignore_exceptions: bool = False,
+                               print_exc: bool = True,
+                               wait_sl_tp_checking: float = 5,
+                               limit: int = 1000,
+                               strategy_in_sleep: bool = False,
+                               deposit_part: float = 1.0,  # for all trades
+                               **strategy_kwargs):
+        can_orders: int = len(tickers)
+        bet_for_trading_on_client_copy: float = bet_for_trading_on_client
+
+        class MultiRealTimeTrader(self.__class__):
+            def get_trading_predict(self,
+                                    bet_for_trading_on_client: float = np.inf,
+                                    coin_lotsize_division: bool = True
+                                    ) -> Dict[str, Union[str, float]]:
+                balance = self.client.get_balance_ticker(self.ticker.split('/')[1])
+                bet = (balance * 10) / (can_orders / deposit_part - brokers.TradingClient.cls_open_orders)
+                bet /= 10  # decimal analog
+                if bet > bet_for_trading_on_client_copy:
+                    bet = bet_for_trading_on_client_copy
+                return super().get_trading_predict(bet_for_trading_on_client=bet,
+                                                   coin_lotsize_division=coin_lotsize_division)
+
+        def start_trading(pare):
+            trader = MultiRealTimeTrader(ticker=pare,
+                                         interval=self.interval,
+                                         trading_on_client=self.trading_on_client)
+            trader.set_pyplot()
+            trader.set_client(copy(self.client))
+
+            while True:
+                if datetime.now() >= start_time:
+                    break
+            trader.realtime_trading(strategy=trader._get_attr(strategy_name),
+                                    ticker=pare,
+                                    print_out=print_out,
+                                    coin_lotsize_division=coin_lotsize_division,
+                                    ignore_exceptions=ignore_exceptions,
+                                    print_exc=print_exc,
+                                    wait_sl_tp_checking=wait_sl_tp_checking,
+                                    limit=limit,
+                                    strategy_in_sleep=strategy_in_sleep,
+                                    **strategy_kwargs)
+
+        for ticker in tickers:
+            thread = Thread(target=start_trading, args=(ticker,))
+            thread.start()
+
     def log_data(self):
         self.fig.update_yaxes(row=1, col=1, type='log')
         utils.logger.debug('trader log data')
@@ -1250,9 +1306,9 @@ winrate: {self.winrate}%"""
         closes: np.ndarray = self.df['Close'].values
         sig: utils.PREDICT_TYPE
         close: float
-        seted: utils.SETED_TYPE
+        seted: utils.CONVERTED_TYPE
         ts: Dict[str, float]
-        for sig, close, seted in zip(self.returns, closes, utils.set_(self.returns)):
+        for sig, close, seted in zip(self.returns, closes, utils.convert(self.returns)):
             if seted is not np.nan:
                 self._open_price = close
                 if set_take or set_stop:
@@ -1343,6 +1399,7 @@ winrate: {self.winrate}%"""
             self.returns.append(flag)
             self._stop_losses.append(flag_stop_loss)
         self.set_credit_leverages()
+        self.set_open_stop_and_take(set_stop=False)
         return self.returns
 
     def find_TBH_TBL(self) -> utils.PREDICT_TYPE_LIST:
@@ -1388,97 +1445,6 @@ winrate: {self.winrate}%"""
         self.set_open_stop_and_take()
         return self.returns
 
-    def is_doji(self) -> List[bool]:
-        """
-        :returns: list of booleans.
-        """
-        ret: List[bool] = []
-        for close, open_ in zip(self.df['Close'].values,
-                                self.df['Open'].values):
-            if close == open_:
-                ret.append(True)
-            else:
-                ret.append(False)
-        self.set_credit_leverages()
-        self.set_open_stop_and_take()
-        return ret
-
-    def find_all_talib_patterns(self):
-        open_ = self.df['Open']
-        high = self.df['High']
-        low = self.df['Low']
-        close = self.df['Close']
-
-        patterns = map(
-            utils.ta_lib_collider_all,
-            [
-                talib.CDL2CROWS(open_, high, low, close),
-                talib.CDL3BLACKCROWS(open_, high, low, close),
-                talib.CDL3INSIDE(open_, high, low, close),
-                talib.CDL3LINESTRIKE(open_, high, low, close),
-                talib.CDL3OUTSIDE(open_, high, low, close),
-                talib.CDL3STARSINSOUTH(open_, high, low, close),
-                talib.CDL3WHITESOLDIERS(open_, high, low, close),
-                talib.CDLABANDONEDBABY(open_, high, low, close),
-                talib.CDLADVANCEBLOCK(open_, high, low, close),
-                talib.CDLBELTHOLD(open_, high, low, close),
-                talib.CDLBREAKAWAY(open_, high, low, close),
-                talib.CDLCLOSINGMARUBOZU(open_, high, low, close),
-                talib.CDLCONCEALBABYSWALL(open_, high, low, close),
-                talib.CDLCOUNTERATTACK(open_, high, low, close),
-                talib.CDLDARKCLOUDCOVER(open_, high, low, close),
-                talib.CDLDOJI(open_, high, low, close),
-                talib.CDLDOJISTAR(open_, high, low, close),
-                talib.CDLDRAGONFLYDOJI(open_, high, low, close),
-                talib.CDLENGULFING(open_, high, low, close),
-                talib.CDLEVENINGDOJISTAR(open_, high, low, close),
-                talib.CDLEVENINGSTAR(open_, high, low, close),
-                talib.CDLGAPSIDESIDEWHITE(open_, high, low, close),
-                talib.CDLGRAVESTONEDOJI(open_, high, low, close),
-                talib.CDLHAMMER(open_, high, low, close),
-                talib.CDLHANGINGMAN(open_, high, low, close),
-                talib.CDLHARAMI(open_, high, low, close),
-                talib.CDLHARAMICROSS(open_, high, low, close),
-                talib.CDLHIGHWAVE(open_, high, low, close),
-                talib.CDLHIKKAKE(open_, high, low, close),
-                talib.CDLHIKKAKEMOD(open_, high, low, close),
-                talib.CDLHOMINGPIGEON(open_, high, low, close),
-                talib.CDLIDENTICAL3CROWS(open_, high, low, close),
-                talib.CDLINNECK(open_, high, low, close),
-                talib.CDLINVERTEDHAMMER(open_, high, low, close),
-                talib.CDLKICKING(open_, high, low, close),
-                talib.CDLKICKINGBYLENGTH(open_, high, low, close),
-                talib.CDLLADDERBOTTOM(open_, high, low, close),
-                talib.CDLLONGLEGGEDDOJI(open_, high, low, close),
-                talib.CDLLONGLINE(open_, high, low, close),
-                talib.CDLMARUBOZU(open_, high, low, close),
-                talib.CDLMATCHINGLOW(open_, high, low, close),
-                talib.CDLMATHOLD(open_, high, low, close),
-                talib.CDLMORNINGDOJISTAR(open_, high, low, close),
-                talib.CDLMORNINGSTAR(open_, high, low, close),
-                talib.CDLONNECK(open_, high, low, close),
-                talib.CDLPIERCING(open_, high, low, close),
-                talib.CDLRICKSHAWMAN(open_, high, low, close),
-                talib.CDLRISEFALL3METHODS(open_, high, low, close),
-                talib.CDLSEPARATINGLINES(open_, high, low, close),
-                talib.CDLSHOOTINGSTAR(open_, high, low, close),
-                talib.CDLSHORTLINE(open_, high, low, close),
-                talib.CDLSPINNINGTOP(open_, high, low, close),
-                talib.CDLSTALLEDPATTERN(open_, high, low, close),
-                talib.CDLSTICKSANDWICH(open_, high, low, close),
-                talib.CDLTAKURI(open_, high, low, close),
-                talib.CDLTASUKIGAP(open_, high, low, close),
-                talib.CDLTHRUSTING(open_, high, low, close),
-                talib.CDLTRISTAR(open_, high, low, close),
-                talib.CDLUNIQUE3RIVER(open_, high, low, close),
-                talib.CDLUPSIDEGAP2CROWS(open_, high, low, close),
-                talib.CDLXSIDEGAP3METHODS(open_, high, low, close)
-            ]
-        )
-
-        patterns = map(utils.anti_set_, patterns)
-        return self.multi_strategy_collider(*patterns, mode='super')
-
     def get_support_resistance(self) -> Dict[str, Dict[int, float]]:
         lows = self.df['Low'].values
         highs = self.df['High'].values
